@@ -2,7 +2,7 @@
 /**
  * Nag_Driver:: defines an API for implementing storage backends for Nag.
  *
- * $Horde: nag/lib/Driver.php,v 1.57.2.18 2008/05/25 13:59:18 jan Exp $
+ * $Horde: nag/lib/Driver.php,v 1.57.2.28 2008/11/26 21:25:25 chuck Exp $
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
@@ -169,7 +169,7 @@ class Nag_Driver {
      */
     function &singleton($tasklist = '', $driver = null, $params = null)
     {
-        static $instances;
+        static $instances = array();
 
         if (is_null($driver)) {
             $driver = $GLOBALS['conf']['storage']['driver'];
@@ -179,10 +179,6 @@ class Nag_Driver {
             $params = Horde::getDriverConfig('storage', $driver);
         }
 
-        if (!isset($instances)) {
-            $instances = array();
-        }
-
         $signature = serialize(array($tasklist, $driver, $params));
         if (!isset($instances[$signature])) {
             $instances[$signature] =& Nag_Driver::factory($tasklist, $driver, $params);
@@ -229,20 +225,25 @@ class Nag_Driver {
         if (is_a($taskId, 'PEAR_Error')) {
             return $taskId;
         }
+        $task = $this->get($taskId);
 
         /* Log the creation of this item in the history log. */
         $history = &Horde_History::singleton();
         $history->log('nag:' . $this->_tasklist . ':' . $uid, array('action' => 'add'), true);
 
+        /* Log completion status changes. */
+        if ($completed) {
+            $history->log('nag:' . $this->_tasklist . ':' . $uid, array('action' => 'complete'), true);
+        }
+
         /* Notify users about the new event. */
-        $result = Nag::sendNotification('add', $this->_tasklist, $name, $desc, $due, $priority);
+        $result = Nag::sendNotification('add', $task);
         if (is_a($result, 'PEAR_Error')) {
             Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
         }
 
         /* Add an alarm if necessary. */
         if (!empty($GLOBALS['conf']['alarms']['driver']) && !empty($alarm)) {
-            $task = $this->get($taskId);
             $alarm = $task->toAlarm();
             if ($alarm) {
                 $alarm['start'] = new Horde_Date($alarm['start']);
@@ -273,11 +274,13 @@ class Nag_Driver {
      * @param string $owner            The owner of the event.
      * @param string $assignee         The assignee of the event.
      * @param integer $completed_date  The task's completion date.
+     * @param string $tasklist         The new tasklist.
      */
     function modify($taskId, $name, $desc, $start = 0, $due = 0, $priority = 0,
                     $estimate = 0.0, $completed = 0, $category = '',
                     $alarm = 0, $parent = '', $private = false,
-                    $owner = null, $assignee = null, $completed_date = null)
+                    $owner = null, $assignee = null, $completed_date = null,
+                    $tasklist = null)
     {
         /* Retrieve unmodified task. */
         $task = $this->get($taskId);
@@ -298,28 +301,6 @@ class Nag_Driver {
             return $modify;
         }
 
-        /* Log the modification of this item in the history log. */
-        if (!empty($task->uid)) {
-            $history = &Horde_History::singleton();
-            $history->log('nag:' . $this->_tasklist . ':' . $task->uid, array('action' => 'modify'), true);
-        }
-
-        /* Log completion status changes. */
-        if ($task->completed != $completed) {
-            $history = &Horde_History::singleton();
-            $attributes = array('action' => 'complete');
-            if (!$completed) {
-                $attributes['ts'] = 0;
-            }
-            $history->log('nag:' . $this->_tasklist . ':' . $task->uid, $attributes, true);
-        }
-
-        /* Notify users about the changed event. */
-        $result = Nag::sendNotification('edit', $this->_tasklist, $name, $desc, $due, $priority);
-        if (is_a($result, 'PEAR_Error')) {
-            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
-        }
-
         /* Update alarm if necessary. */
         if (!empty($GLOBALS['conf']['alarms']['driver'])) {
             require_once 'Horde/Alarm.php';
@@ -336,9 +317,70 @@ class Nag_Driver {
             }
         }
 
+        $new_task = $this->get($task->id);
+        $log_tasklist = $this->_tasklist;
+        if (!is_null($tasklist) && $task->tasklist != $tasklist) {
+            /* Moving the task to another tasklist. */
+            $share = $GLOBALS['nag_shares']->getShare($task->tasklist);
+            if (is_a($share, 'PEAR_Error')) {
+                return $share;
+            }
+
+            if (!$share->hasPermission(Auth::getAuth(), PERMS_DELETE)) {
+                $GLOBALS['notification']->push(sprintf(_("Access denied removing task from %s."), $share->get('name')), 'horde.error');
+                return false;
+            }
+
+            $share = $GLOBALS['nag_shares']->getShare($tasklist);
+            if (is_a($share, 'PEAR_Error')) {
+                return $share;
+            }
+
+            if (!$share->hasPermission(Auth::getAuth(), PERMS_EDIT)) {
+                $GLOBALS['notification']->push(sprintf(_("Access denied moving the task to %s."), $share->get('name')), 'horde.error');
+            }
+
+            $moved = $this->_move($task->id, $tasklist);
+            if (is_a($moved, 'PEAR_Error')) {
+                return $moved;
+            }
+            $new_storage = &Nag_Driver::singleton($tasklist);
+            $new_task = $new_storage->get($task->id);
+
+            /* Log the moving of this item in the history log. */
+            if (!empty($task->uid)) {
+                $history = &Horde_History::singleton();
+                $history->log('nag:' . $task->tasklist . ':' . $task->uid, array('action' => 'delete'), true);
+                $history->log('nag:' . $tasklist . ':' . $task->uid, array('action' => 'add'), true);
+                $log_tasklist = $tasklist;
+            }
+        }
+
+        /* Log the modification of this item in the history log. */
+        if (!empty($task->uid)) {
+            $history = &Horde_History::singleton();
+            $history->log('nag:' . $log_tasklist . ':' . $task->uid, array('action' => 'modify'), true);
+        }
+
+        /* Log completion status changes. */
+        if ($task->completed != $completed) {
+            $history = &Horde_History::singleton();
+            $attributes = array('action' => 'complete');
+            if (!$completed) {
+                $attributes['ts'] = 0;
+            }
+            $history->log('nag:' . $log_tasklist . ':' . $task->uid, $attributes, true);
+        }
+
+        /* Notify users about the changed event. */
+        $result = Nag::sendNotification('edit', $new_task, $task);
+        if (is_a($result, 'PEAR_Error')) {
+            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
+        }
+
         return true;
     }
-
+    
     /**
      * Deletes a task and handles notification.
      *
@@ -361,7 +403,7 @@ class Nag_Driver {
         }
 
         /* Notify users about the deleted event. */
-        $result = Nag::sendNotification('delete', $this->_tasklist, $task->name, $task->desc, $task->due, $task->priority);
+        $result = Nag::sendNotification('delete', $task);
         if (is_a($result, 'PEAR_Error')) {
             Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
         }
@@ -687,6 +729,20 @@ class Nag_Task {
                                 $this->completed_date);
     }
 
+    /**
+     * Returns the parent task of this task, if one exists.
+     *
+     * @return Nag_Task  The parent task, null if none exists, PEAR_Error on
+     *                   failure.
+     */
+    function getParent()
+    {
+        if (!$this->parent_id) {
+            return null;
+        }
+        return Nag::getTask($this->tasklist, $this->parent_id);
+    }
+
     /**
      * Adds a sub task to this task.
      *
@@ -816,6 +872,19 @@ class Nag_Task {
         return $estimate;
     }
 
+    /**
+     * Format the description - link URLs, etc.
+     *
+     * @return string
+     */
+    function getFormattedDescription()
+    {
+        require_once 'Horde/Text/Filter.php';
+        $desc = Text_Filter::filter($this->desc, 'text2html', array('parselevel' => TEXT_HTML_MICRO));
+        $desc = Horde::callHook('_nag_hook_format_description', array($desc), 'nag', $desc);
+        return $desc;
+    }
+
     /**
      * Resets the tasks iterator.
      *
@@ -966,6 +1035,8 @@ class Nag_Task {
             NAG_SORT_CATEGORY => 'ByCategory',
             NAG_SORT_DUE => 'ByDue',
             NAG_SORT_COMPLETION => 'ByCompletion',
+            NAG_SORT_ASSIGNEE => 'ByAssignee',
+            NAG_SORT_ESTIMATE => 'ByEstimate',
             NAG_SORT_OWNER => 'ByOwner'
         );
 
@@ -1114,8 +1185,6 @@ class Nag_Task {
 
         if (!empty($this->assignee)) {
             $vTodo->setAttribute('ORGANIZER', $this->assignee);
-        } elseif (!empty($this->owner)) {
-            $vTodo->setAttribute('ORGANIZER', $this->owner);
         }
 
         if (!empty($this->name)) {
