@@ -2,9 +2,9 @@
 /**
  * Nag driver classes for the Kolab IMAP server.
  *
- * $Horde: nag/lib/Driver/kolab.php,v 1.6.10.13 2008/01/02 11:32:30 jan Exp $
+ * $Horde: nag/lib/Driver/kolab.php,v 1.6.10.16 2009/01/06 15:25:06 jan Exp $
  *
- * Copyright 2004-2008 The Horde Project (http://www.horde.org/)
+ * Copyright 2004-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
@@ -165,7 +165,7 @@ class Nag_Driver_kolab extends Nag_Driver {
      * @param string $taskId       The task to move.
      * @param string $newTasklist  The new tasklist.
      */
-    function move($taskId, $newTasklist)
+    function _move($taskId, $newTasklist)
     {
         return $this->_wrapper->move($taskId, $newTasklist);
     }
@@ -820,7 +820,7 @@ class Nag_Driver_kolab_wrapper_new extends Nag_Driver_kolab_wrapper {
                                           'alarm' => $alarm,
                                           'parent' => $parent,
                                           'sensitivity' => $sensitivity,
-                                          'estimate' => (int) $estimate,
+                                          'estimate' => $estimate,
                                           'completed_date' => $completed_date,
                                           'creator' => array(
                                               'smpt-address' => $owner,
