@@ -30,6 +30,16 @@ define('NAG_SORT_CATEGORY', 'category');
  */
 define('NAG_SORT_OWNER', 'tasklist');
 
+/**
+ * Sort by estimate.
+ */
+define('NAG_SORT_ESTIMATE', 'estimate');
+
+/**
+ * Sort by assignee.
+ */
+define('NAG_SORT_ASSIGNEE', 'assignee');
+
 /**
  * Sort in ascending order.
  */
@@ -43,7 +53,7 @@ define('NAG_SORT_DESCEND', 1);
 /**
  * Nag Base Class.
  *
- * $Horde: nag/lib/Nag.php,v 1.124.2.24 2008/05/25 17:03:12 jan Exp $
+ * $Horde: nag/lib/Nag.php,v 1.124.2.33 2009/01/13 15:46:34 chuck Exp $
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
@@ -90,9 +100,7 @@ class Nag {
      *
      * This function will also sort the resulting list, if requested.
      *
-     * @param string $sortby      The field by which to sort
-     *                            (NAG_SORT_PRIORITY, NAG_SORT_NAME
-     *                            NAG_SORT_DUE, NAG_SORT_COMPLETION).
+     * @param string $sortby      The field by which to sort (NAG_SORT_*).
      * @param integer $sortdir    The direction by which to sort
      *                            (NAG_SORT_ASCEND, NAG_SORT_DESCEND).
      * @param string $altsortby   The secondary sort field.
@@ -157,6 +165,9 @@ class Nag {
                 if ($app != 'nag' &&
                     $registry->hasMethod('getListTypes', $app)) {
                     $types = $registry->callByPackage($app, 'getListTypes');
+                    if (is_a($types, 'PEAR_Error')) {
+                        continue;
+                    }
                     if (!empty($types['taskHash'])) {
                         $newtasks = $registry->callByPackage($app, 'listAs', array('taskHash'));
                         if (is_a($newtasks, 'PEAR_Error')) {
@@ -271,7 +282,13 @@ class Nag {
      */
     function listTasklists($owneronly = false, $permission = PERMS_SHOW)
     {
-        $tasklists = $GLOBALS['nag_shares']->listShares(Auth::getAuth(), $permission, $owneronly ? Auth::getAuth() : null, 0, 0, 'name');
+        // Work around BC break in listShares() parameters (http://bugs.horde.org/ticket/7820)
+        include_once $GLOBALS['registry']->get('fileroot', 'horde') . '/lib/version.php';
+        if (version_compare(HORDE_VERSION, '3.2', '<')) {
+            $tasklists = $GLOBALS['nag_shares']->listShares(Auth::getAuth(), $permission, $owneronly ? Auth::getAuth() : null, DATATREE_ROOT, true, 0, 0, 'name');
+        } else {
+            $tasklists = $GLOBALS['nag_shares']->listShares(Auth::getAuth(), $permission, $owneronly ? Auth::getAuth() : null, 0, 0, 'name');
+        }
         if (is_a($tasklists, 'PEAR_Error')) {
             Horde::logMessage($tasklists, __FILE__, __LINE__, PEAR_LOG_ERR);
             return array();
@@ -452,6 +469,40 @@ class Nag {
         return $alarm_text;
     }
 
+    /**
+     * Returns the full name and a compose to message an assignee.
+     *
+     * @param string $assignee  The assignee's user name.
+     * @param boolean $link     Whether to link to an email compose screen.
+     *
+     * @return string  The formatted assignee name.
+     */
+    function formatAssignee($assignee, $link = false)
+    {
+        if (!strlen($assignee)) {
+            return '';
+        }
+
+        require_once 'Horde/Identity.php';
+        $identity = &Identity::singleton('none', $assignee);
+        $fullname = $identity->getValue('fullname');
+        if (!strlen($fullname)) {
+            $fullname = $assignee;
+        }
+        $email = $identity->getValue('from_addr');
+        if ($link && !empty($email) &&
+            $GLOBALS['registry']->hasMethod('mail/compose')) {
+            return Horde::link($GLOBALS['registry']->call(
+                                   'mail/compose',
+                                   array(array('to' => $email))))
+                . @htmlspecialchars($fullname . ' <' . $email . '>',
+                                    ENT_COMPAT, NLS::getCharset())
+                . '</a>';
+        } else {
+            return @htmlspecialchars($fullname, ENT_COMPAT, NLS::getCharset());
+        }
+    }
+
     /**
      * Returns the specified permission for the current user.
      *
@@ -642,62 +693,39 @@ class Nag {
      * Sends email notifications that a task has been added, edited, or
      * deleted to users that want such notifications.
      *
-     * @param string $action     The event action. One of "add", "edit", or
-     *                           "delete".
-     * @param string $tasklist   The tasklist of the task we are dealing with.
-     * @param string $name       The name (short) of the task.
-     * @param string $desc       The description (long) of the task.
-     * @param integer $due       The due date of the task.
-     * @param integer $priority  The priority of the task.
+     * @param string $action      The event action. One of "add", "edit", or
+     *                            "delete".
+     * @param Nag_Task $task      The changed task.
+     * @param Nag_Task $old_task  The original task if $action is "edit".
      */
-    function sendNotification($action, $tasklist, $name, $desc, $due, $priority)
+    function sendNotification($action, $task, $old_task = null)
     {
-        global $conf;
-
-        switch ($action) {
-        case 'add':
-            $subject = _("Task added:");
-            $notification_message = _("You requested to be notified when tasks are added to your tasklists.") . "\n\n" . _("The task \"%s\" has been added to \"%s\" tasklist, with a due date of: %s.");
-            break;
-
-        case 'edit':
-            $subject = _("Task modified:");
-            $notification_message = _("You requested to be notified when tasks are edited on your tasklists.") . "\n\n" . _("The task \"%s\" has been edited on \"%s\" tasklist, with a due date of: %s.");
-            break;
-
-        case 'delete':
-            $subject = _("Task deleted:");
-            $notification_message = _("You requested to be notified when tasks are deleted from your tasklists.") . "\n\n" . _("The task \"%s\" has been deleted from \"%s\" tasklist, with a due date of: %s.");
-            break;
-
-        default:
+        if (!in_array($action, array('add', 'edit', 'delete'))) {
             return PEAR::raiseError('Unknown event action: ' . $action);
         }
 
+        $share = &$GLOBALS['nag_shares']->getShare($task->tasklist);
+        if (is_a($share, 'PEAR_Error')) {
+            return $share;
+        }
+
         require_once 'Horde/Group.php';
         require_once 'Horde/Identity.php';
         require_once 'Horde/MIME.php';
         require_once 'Horde/MIME/Headers.php';
         require_once 'Horde/MIME/Message.php';
 
-        $share = &$GLOBALS['nag_shares']->getShare($tasklist);
-        if (is_a($share, 'PEAR_Error')) {
-            return $share;
-        }
-
         $groups = &Group::singleton();
         $recipients = array();
         $identity = &Identity::singleton();
         $from = $identity->getDefaultFromAddress(true);
 
         $owner = $share->get('owner');
-        if (Nag::_notificationPref($owner, 'owner')) {
-            $recipients[$owner] = true;
-        }
+        $recipients[$owner] = Nag::_notificationPref($owner, 'owner');
 
         foreach ($share->listUsers(PERMS_READ) as $user) {
-            if (!isset($recipients[$user])) {
-                $recipients[$user] = Nag::_notificationPref($user, 'read', $tasklist);
+            if (empty($recipients[$user])) {
+                $recipients[$user] = Nag::_notificationPref($user, 'read', $task->tasklist);
             }
         }
         foreach ($share->listGroups(PERMS_READ) as $group) {
@@ -711,53 +739,199 @@ class Nag {
                 continue;
             }
             foreach ($group_users as $user) {
-                if (!isset($recipients[$user])) {
-                    $recipients[$user] = Nag::_notificationPref($user, 'read', $tasklist);
+                if (empty($recipients[$user])) {
+                    $recipients[$user] = Nag::_notificationPref($user, 'read', $task->tasklist);
                 }
             }
         }
 
         $addresses = array();
-        foreach ($recipients as $user => $send) {
-            if ($send) {
-                $identity = &Identity::singleton('none', $user);
-                $email = $identity->getValue('from_addr');
-                if (strstr($email, '@')) {
-                    list($mailbox, $host) = explode('@', $email);
-                    $addresses[] = MIME::rfc822WriteAddress($mailbox, $host, $identity->getValue('fullname'));
-                }
+        foreach ($recipients as $user => $vals) {
+            if (!$vals) {
+                continue;
             }
+            $identity = &Identity::singleton('none', $user);
+            $email = $identity->getValue('from_addr');
+            if (strpos($email, '@') === false) {
+                continue;
+            }
+            list($mailbox, $host) = explode('@', $email);
+            if (!isset($addresses[$vals['lang']][$vals['tf']][$vals['df']])) {
+                $addresses[$vals['lang']][$vals['tf']][$vals['df']] = array();
+            }
+            $addresses[$vals['lang']][$vals['tf']][$vals['df']][] = MIME::rfc822WriteAddress($mailbox, $host, $identity->getValue('fullname'));
         }
 
-        if (!count($addresses)) {
+        if (!$addresses) {
             return;
         }
 
+        $mail_driver = $GLOBALS['conf']['mailer']['type'];
+        $mail_params = $GLOBALS['conf']['mailer']['params'];
+        if ($mail_driver == 'smtp' && $mail_params['auth'] &&
+            empty($mail_params['username'])) {
+            $mail_params['username'] = Auth::getAuth();
+            $mail_params['password'] = Auth::getCredential('password');
+        }
+
         $msg_headers = new MIME_Headers();
         $msg_headers->addMessageIdHeader();
         $msg_headers->addAgentHeader();
         $msg_headers->addHeader('Date', date('r'));
         $msg_headers->addHeader('From', $from);
-        $msg_headers->addHeader('Subject', $subject . ' ' . $name);
 
-        $message = "\n" . sprintf($notification_message, $name, $share->get('name'), $due ? strftime('%x %X', $due) : 'no due date' . "\n\n" . $desc);
+        foreach ($addresses as $lang => $twentyFour) {
+            NLS::setLang($lang);
 
-        $mime = new MIME_Message();
-        $body = new MIME_Part('text/plain', String::wrap($message, 76, "\n"), NLS::getCharset());
+            $view_link = Util::addParameter(Horde::applicationUrl('view.php', true),
+                                            array('tasklist' => $task->tasklist,
+                                                  'task' => $task->id),
+                                            null, false);
 
-        $mime->addPart($body);
-        $msg_headers->addMIMEHeaders($mime);
+            switch ($action) {
+            case 'add':
+                $subject = _("Task added:");
+                $notification_message = _("You requested to be notified when tasks are added to your task lists.")
+                    . "\n\n"
+                    . _("The task \"%s\" has been added to task list \"%s\", with a due date of: %s.")
+                    . "\n"
+                    . str_replace("%","%%",$view_link);
+                break;
 
-        $mail_driver = $conf['mailer']['type'];
-        $mail_params = $conf['mailer']['params'];
-        if ($mail_driver == 'smtp' && $mail_params['auth'] &&
-            empty($mail_params['username'])) {
-            $mail_params['username'] = Auth::getAuth();
-            $mail_params['password'] = Auth::getCredential('password');
+            case 'edit':
+                $subject = _("Task modified:");
+                $notification_message = _("You requested to be notified when tasks are edited on your task lists.")
+                    . "\n\n"
+                    . _("The task \"%s\" has been edited on task list \"%s\".")
+                    . "\n"
+                    . str_replace("%","%%",$view_link)
+                    . "\n\n"
+                    . _("Changes made for this task:");
+                if ($old_task->name != $task->name) {
+                    $notification_message .= "\n - "
+                        . sprintf(_("Changed name from \"%s\" to \"%s\""),
+                                  $old_task->name, $task->name);
+                }
+                if ($old_task->tasklist != $task->tasklist) {
+                    $old_share = &$GLOBALS['nag_shares']->getShare($old_task->tasklist);
+                    $notification_message .= "\n - "
+                        . sprintf(_("Changed task list from \"%s\" to \"%s\""),
+                                  $old_share->get('name'), $share->get('name'));
+                }
+                if ($old_task->parent_id != $task->parent_id) {
+                    $old_parent = $old_task->getParent();
+                    if (!is_a($old_parent, 'PEAR_Error')) {
+                        $parent = $task->getParent();
+                        if (!is_a($parent, 'PEAR_Error')) {
+                            $notification_message .= "\n - "
+                                . sprintf(_("Changed parent task from \"%s\" to \"%s\""),
+                                          $old_parent ? $old_parent->name : _("no parent"),
+                                          $parent ? $parent->name : _("no parent"));
+                        }
+                    }
+                }
+                if ($old_task->category != $task->category) {
+                    $notification_message .= "\n - "
+                        . sprintf(_("Changed category from \"%s\" to \"%s\""),
+                                  $old_task->category, $task->category);
+                }
+                if ($old_task->assignee != $task->assignee) {
+                    require_once 'Horde/Identity.php';
+                    $identity = &Identity::singleton('none', $old_task->assignee);
+                    $old_name = $identity->getValue('fullname');
+                    if (!strlen($old_name)) {
+                        $old_name = $old_task->assignee;
+                    }
+                    $identity = &Identity::singleton('none', $task->assignee);
+                    $new_name = $identity->getValue('fullname');
+                    if (!strlen($new_name)) {
+                        $new_name = $new_task->assignee;
+                    }
+                    $notification_message .= "\n - "
+                        . sprintf(_("Changed assignee from \"%s\" to \"%s\""),
+                                  $old_name, $new_name);
+                }
+                if ($old_task->private != $task->private) {
+                    $notification_message .= "\n - "
+                        . ($task->private ? _("Turned privacy on") : _("Turned privacy off"));
+                }
+                if ($old_task->due != $task->due) {
+                    $notification_message .= "\n - "
+                        . sprintf(_("Changed due date from %s to %s"),
+                                  $old_task->due ? Nag::formatDate($old_task->due) : _("no due date"),
+                                  $task->due ? Nag::formatDate($task->due) : _("no due date"));
+                }
+                if ($old_task->start != $task->start) {
+                    $notification_message .= "\n - "
+                        . sprintf(_("Changed start date from %s to %s"),
+                                  $old_task->start ? Nag::formatDate($old_task->start) : _("no start date"),
+                                  $task->start ? Nag::formatDate($task->start) : _("no start date"));
+                }
+                if ($old_task->alarm != $task->alarm) {
+                    $notification_message .= "\n - "
+                        . sprintf(_("Changed alarm from %s to %s"),
+                                  Nag::formatAlarm($old_task->alarm), Nag::formatAlarm($task->alarm));
+                }
+                if ($old_task->priority != $task->priority) {
+                    $notification_message .= "\n - "
+                        . sprintf(_("Changed priority from %s to %s"),
+                                  $old_task->priority, $task->priority);
+                }
+                if ($old_task->estimate != $task->estimate) {
+                    $notification_message .= "\n - "
+                        . sprintf(_("Changed estimate from %s to %s"),
+                                  $old_task->estimate, $task->estimate);
+                }
+                if ($old_task->completed != $task->completed) {
+                    $notification_message .= "\n - "
+                        . sprintf(_("Changed completion from %s to %s"),
+                                  $old_task->completed ? _("completed") : _("not completed"),
+                                  $task->completed ? _("completed") : _("not completed"));
+                }
+                if ($old_task->desc != $task->desc) {
+                    $notification_message .= "\n - " . _("Changed description");
+                }
+                break;
+
+            case 'delete':
+                $subject = _("Task deleted:");
+                $notification_message =
+                    _("You requested to be notified when tasks are deleted from your task lists.")
+                    . "\n\n"
+                    . _("The task \"%s\" has been deleted from task list \"%s\".");
+                break;
+            }
+
+            $msg_headers->removeHeader('Subject');
+            $msg_headers->addHeader('Subject', $subject . ' ' . $task->name);
+
+            foreach ($twentyFour as $tf => $dateFormat) {
+                foreach ($dateFormat as $df => $df_recipients) {
+                    $message = sprintf($notification_message,
+                                       $task->name,
+                                       $share->get('name'),
+                                       $task->due ? strftime($df, $task->due) . ' ' . date($tf ? 'H:i' : 'h:ia', $task->due) : _("no due date"));
+                    if (strlen(trim($task->desc))) {
+                        $message .= "\n\n" . _("Task description:") . "\n\n" . $task->desc;
+                    }
+
+                    $mime = new MIME_Message();
+                    $body = new MIME_Part('text/plain', String::wrap($message, 76, "\n"), NLS::getCharset());
+
+                    $mime->addPart($body);
+                    $msg_headers->addMIMEHeaders($mime);
+
+                    Horde::logMessage(sprintf('Sending event notifications for %s to %s',
+                                              $task->name, implode(', ', $df_recipients)),
+                                      __FILE__, __LINE__, PEAR_LOG_INFO);
+                    $sent = $mime->send(implode(', ', $df_recipients), $msg_headers,
+                                        $mail_driver, $mail_params);
+                    if (is_a($sent, 'PEAR_Error')) {
+                        return $sent;
+                    }
+                }
+            }
         }
-
-        Horde::logMessage(sprintf('Sending event notifications for %s to %s', $name, implode(', ', $addresses)), __FILE__, __LINE__, PEAR_LOG_DEBUG);
-        return $mime->send(implode(', ', $addresses), $msg_headers, $mail_driver, $mail_params);
     }
 
     /**
@@ -808,17 +982,25 @@ class Nag {
                                    'nag', $user, '', null,
                                    false);
         $prefs->retrieve();
+        $vals = array('lang' => $prefs->getValue('language'),
+                      'tf' => $prefs->getValue('twentyFour'),
+                      'df' => $prefs->getValue('date_format'));
+
+        if ($prefs->getValue('task_notification_exclude_self') &&
+            $user == Auth::getAuth()) {
+            return false;
+        }
 
         $notification = $prefs->getValue('task_notification');
         switch ($notification) {
         case 'owner':
-            return $mode == 'owner';
+            return $mode == 'owner' ? $vals : false;
         case 'read':
-            return $mode == 'read';
+            return $mode == 'read' ? $vals : false;
         case 'show':
             if ($mode == 'read') {
                 $display_tasklists = unserialize($prefs->getValue('display_tasklists'));
-                return in_array($tasklist, $display_tasklists);
+                return in_array($tasklist, $display_tasklists) ? $vals : false;;
             }
         }
 
@@ -887,6 +1069,72 @@ class Nag {
         return strcasecmp($b->name, $a->name);
     }
 
+    /**
+     * Comparison function for sorting tasks by assignee.
+     *
+     * @param array $a  Task one.
+     * @param array $b  Task two.
+     *
+     * @return integer  1 if task one is greater, -1 if task two is greater;
+     *                  0 if they are equal.
+     */
+    function _sortByAssignee($a, $b)
+    {
+        return strcasecmp($a->assignee, $b->assignee);
+    }
+
+    /**
+     * Comparison function for reverse sorting tasks by assignee.
+     *
+     * @param array $a  Task one.
+     * @param array $b  Task two.
+     *
+     * @return integer  -1 if task one is greater, 1 if task two is greater;
+     *                  0 if they are equal.
+     */
+    function _rsortByAssignee($a, $b)
+    {
+        return strcasecmp($b->assignee, $a->assignee);
+    }
+
+    /**
+     * Comparison function for sorting tasks by assignee.
+     *
+     * @param array $a  Task one.
+     * @param array $b  Task two.
+     *
+     * @return integer  1 if task one is greater, -1 if task two is greater;
+     *                  0 if they are equal.
+     */
+    function _sortByEstimate($a, $b)
+    {
+        $a_est = $a->estimation();
+        $b_est = $b->estimation();
+        if ($a_est == $b_est) {
+            return 0;
+        }
+        return ($a_est > $b_est) ? 1 : -1;
+    }
+
+    /**
+     * Comparison function for reverse sorting tasks by name.
+     *
+     * @param array $a  Task one.
+     * @param array $b  Task two.
+     *
+     * @return integer  -1 if task one is greater, 1 if task two is greater;
+     *                  0 if they are equal.
+     */
+    function _rsortByEstimate($a, $b)
+    {
+        $a_est = $a->estimation();
+        $b_est = $b->estimation();
+        if ($a_est == $b_est) {
+            return 0;
+        }
+        return ($a_est > $b_est) ? -1 : 1;
+    }
+
     /**
      * Comparison function for sorting tasks by category.
      *
